{ apu super I/O init, GPIO test }

unit nct5104;

{$g+,r-,s- enable 286 instructions }

{ pd 131104 initial code

  Note that the pull-up on g02 (R288) should be removed = port 80 disabled.
  Otherwise port G05 will misbehave.
}

interface

uses testutil;

procedure setsio;
procedure testgpio;

implementation

const
  ix=$2e;
  dat=$2f;

{ set a register }

procedure cfg(index,data:byte);
begin
  port[ix]:=index;
  port[dat]:=data;
end;

{ configure super I/O }
{ init the super I/O since Coreboot doesn't enable second UART }

procedure setsio;
begin
  port[ix]:=$87;	{ enter extended function mode }
  port[ix]:=$87;

  cfg($07,$03);		{ device 3 = UART B }
  cfg($60,$02);		{ base H }
  cfg($61,$f8);		{ base L }
  cfg($70,$03);		{ IRQ 3 }
  cfg($30,$01);		{ enable }

			{ &&& this doesn't really work... }
  cfg($07,$14);		{ device 14 = port 80 }
  cfg($e0,$00);		{ disable transmit, otherwise G05 misbehaves }

(*  port[ix]:=$aa;	{ exit extended function mode }
  port[ix]:=$aa; *)	{ leave it open ! }
end;

{ set GPIO pin as input }

var
  dir0,dir1,dat0,dat1:byte;

procedure g_in(num:byte);
begin
  if num<10 then begin
    dir0:=dir0 or (1 shl num);
    cfg($e0,dir0);
  end else begin
    dir1:=dir1 or (1 shl (num-10));
    cfg($e4,dir1);
  end;
end;

{ set GPIO pin as output }

procedure g_out(num:byte);
begin
  if num<10 then begin
    dir0:=dir0 and not (1 shl num);
    cfg($e0,dir0);
  end else begin
    dir1:=dir1 and not (1 shl (num-10));
    cfg($e4,dir1);
  end;
end;

{ set GPIO pin low }

procedure g_0(num:byte);
begin
  if num<10 then begin
    dat0:=dat0 and not (1 shl num);
    cfg($e1,dat0);
  end else begin
    dat1:=dat1 and not (1 shl (num-10));
    cfg($e5,dir1);
  end;
end;

{ set GPIO pin high }

procedure g_1(num:byte);
begin
  if num<10 then begin
    dat0:=dat0 or (1 shl num);
    cfg($e1,dat0);
  end else begin
    dat1:=dat1 or (1 shl (num-10));
    cfg($e5,dir1);
  end;
end;

{ expect GPIO pin status }

procedure g_chk(num,val:byte);
var
  b,mask:byte;
begin
  if num<10 then begin
    port[ix]:=$e1;
    b:=(port[dat] shr num) and 1;
  end else begin
    port[ix]:=$e5;
    b:=(port[dat] shr (num - 10)) and 1;
  end;
  if b<>val then begin
    error:=true;
    write('g',num,'=',b,' ');
  end;
end;

procedure testgpio;
begin
  write('GPIO test: ');
  error:=false;

  port[ix]:=$87;	{ enter extended function mode }
  port[ix]:=$87;

  cfg($07,$0f);		{ access GPIO 0F }
  cfg($e0,$00);		{ G0 push-pull }
  cfg($e1,$00);		{ G1 push-pull }

  cfg($07,$07);		{ access GPIO }
  cfg($30,$03);		{ enable GPIO0, GPIO1 }

  dir0:=$ff; dir1:=$ff;	{ pin directions, 1 = input }
  cfg($e0,dir0);
  cfg($e4,dir1);

  dat0:=$ff; dat1:=$ff;	{ pin status = drive high}
  cfg($e1,dat0);
  cfg($e5,dat1);

  g_chk(00,0);		{ expect pull-down }
  g_chk(01,0);
  g_chk(02,0);
  g_chk(03,0);
  g_chk(04,0);
  g_chk(05,0);
  g_chk(06,0);
  g_chk(07,0);
  g_chk(10,0);		
  g_chk(11,0);
  g_chk(12,0);
  g_chk(13,0);
  g_chk(14,0);
  g_chk(15,0);
  g_chk(16,0);
  g_chk(17,0);

  g_out(00);		{ enable output, driven high }
  g_chk(01,1);		{ check if connected pin is high }
  g_chk(02,0);		{ check if neighboring pin is low }
  g_in(00);		{ disable output }

  g_out(01); g_chk(00,1); g_chk(03,0);              g_in(01);
  g_out(02); g_chk(03,1); g_chk(00,0); g_chk(04,0); g_in(02);
  g_out(03); g_chk(02,1); g_chk(01,0); g_chk(05,0); g_in(03);
  g_out(04); g_chk(05,1); g_chk(02,0); g_chk(06,0); g_in(04);
  g_out(05); g_chk(04,1); g_chk(03,0); g_chk(07,0); g_in(05);
  g_out(06); g_chk(07,1); g_chk(04,0); g_chk(10,0); g_in(06);
  g_out(07); g_chk(06,1); g_chk(05,0); g_chk(11,0); g_in(07);
  g_out(10); g_chk(11,1); g_chk(06,0); g_chk(12,0); g_in(10);
  g_out(11); g_chk(10,1); g_chk(07,0); g_chk(13,0); g_in(11);
  g_out(12); g_chk(13,1); g_chk(10,0); g_chk(14,0); g_in(12);
  g_out(13); g_chk(12,1); g_chk(11,0); g_chk(15,0); g_in(13);
  g_out(14); g_chk(15,1); g_chk(12,0); g_chk(16,0); g_in(14);
  g_out(15); g_chk(14,1); g_chk(13,0); g_chk(17,0); g_in(15);
  g_out(16); g_chk(17,1); g_chk(14,0);              g_in(16);
  g_out(17); g_chk(16,1); g_chk(15,0);              g_in(17);

  if error then fail else ok;
end;

end.
